@@ -14,8 +14,11 @@ import logging; logger = logging.getLogger("morse." + __name__)
 
 from morse.core import blenderapi
 import math
+import imp
 from mathutils import Matrix
 
+imp.reload(blenderapi)
+
 AZERTY = False
 
 # use different keys for QWERTZ/QWERTY or AZERTY keyboards
@@ -186,7 +189,7 @@ def set_human_animation(contr):
 
     scene = blenderapi.scene()
     active_camera = scene.active_camera
-    human = scene.objects[armature.parent.name + suffix]
+    human = scene.objects[armature.parent.name]
 
     # if the human is external, do nothing
     if human.get('External_Robot_Tag') or human['disable_keyboard_control']:
