@@ -6,7 +6,7 @@ import copy
 from morse.builder import bpymorse
 from morse.builder.data import *
 
-from morse.helpers.loading import load_module_attribute
+from morse.helpers.loading import get_class, load_module_attribute
 
 class Configuration(object):
     datastream = {}
@@ -49,6 +49,24 @@ class Configuration(object):
     def link_overlay(component,  manager, overlay_cfg, kwargs):
         Configuration.overlay.setdefault(manager, {})[component.name] = [overlay_cfg, kwargs]
 
+    def has_datastream_configuration(component, stream):
+        try:
+            confs = Configuration.datastream[component.name]
+            for conf in confs:
+                if conf[0] == MORSE_DATASTREAM_MODULE[stream]:
+                    return True
+        except KeyError:
+            return False
+
+    def has_service_configuration(component, stream):
+        try:
+            confs = Configuration.service[component.name]
+            for conf in confs:
+                if conf[0] == MORSE_SERVICE_DICT[stream]:
+                    return True
+        except KeyError:
+            return False
+
     def write_config():
         """ Write the 'component_config.py' file with the supplied settings """
         if not 'component_config.py' in bpymorse.get_texts().keys():
@@ -99,7 +117,6 @@ class AbstractComponent(object):
         obj.parent = self
         self.children.append(obj)
 
-        #TODO: replace by sys._getframes() ??
         import inspect
         try:
             frame = inspect.currentframe()
@@ -114,10 +131,38 @@ class AbstractComponent(object):
                 if component == obj:
                     if not component.basename: # do automatic renaming only if a name is not already manually set
                         component.basename = name
+
         finally:
             del builderscript_frame
             del frame
 
+
+    @staticmethod
+    def close_context(level = 1):
+        import inspect
+        try:
+            frame = inspect.currentframe()
+            builderscript_frame = inspect.getouterframes(frame)[level][0] # parent frame
+            cmpts = builderscript_frame.f_locals
+
+            for name, component in cmpts.items():
+                if isinstance(component, AbstractComponent):
+
+                    if hasattr(component, "parent"):
+                        continue
+
+                    # do automatic renaming only if a name is not already manually set
+                    # component.name accessor set both basename and bpy.name,
+                    # which is the correct behaviour here. The bpy_name may be
+                    # rewritten by _rename_tree, to get the correct hierarcy.
+                    if not component.basename: 
+                        component.name = name
+
+        finally:
+            del builderscript_frame
+            del frame
+
+
     @property
     def name(self):
         return self._bpy_object.name
@@ -190,60 +235,29 @@ class AbstractComponent(object):
             self.properties(my_clock = timer(5.0), my_speed = int(5/2))
 
         """
-        prop = self._bpy_object.game.properties
-        for key in kwargs.keys():
-            if key in prop.keys():
-                self._property_set(key, kwargs[key])
-            else:
-                self._property_new(key, kwargs[key])
-
-    def _property_new(self, name, value, ptype=None):
-        """ Add a new game property for the Blender object
-
-        :param name: property name (string)
-        :param value: property value
-        :param ptype: property type (enum in ['BOOL', 'INT', 'FLOAT', 'STRING', 'TIMER'],
-                      optional, auto-detect, default=None)
-        """
-        self.select()
-        bpymorse.new_game_property()
-        prop = self._bpy_object.game.properties
-        # select the last property in the list (which is the one we just added)
-        prop[-1].name = name
-        return self._property_set(-1, value, ptype)
-
-    def _property_set(self, pid, value, ptype=None):
-        """ Really set the property for the property referenced by pid
-
-        :param pid: the index of property
-        :param value: the property value
-        :param ptype: property type (enum in ['BOOL', 'INT', 'FLOAT', 'STRING', 'TIMER'],
-                      optional, auto-detect, default=None)
-        """
-        prop = self._bpy_object.game.properties
-        if ptype == None:
-            # Detect the type (class name upper case)
-            ptype = value.__class__.__name__.upper()
-        if ptype == 'STR':
-            # Blender property string are called 'STRING' (and not 'str' as in Python)
-            ptype = 'STRING'
-        prop[pid].type = ptype
-        prop[pid].value = value
-        return prop[pid]
+        bpymorse.properties(self._bpy_object, **kwargs)
 
     def select(self):
         bpymorse.select_only(self._bpy_object)
 
-    def get_child(self, name, objects=None):
-        """ get_child returns the child named :param name: """
-        if not objects:
+    def get_child(self, name, objects=None, recursive=True):
+        """ get_child returns the child named :param name: 
+        
+        If several children match the name, a warning is printed and
+        the first one is returned.
+
+        :param name: the textual name of the child
+        :param objects: if specified, look for the child in this list of bpy Objects
+        :param recursive: (default: True) if true, search for the child recursively
+        """
+        if objects is None:
             objects = self._bpy_object.children
         for obj in objects:
             if obj.name == name:
                 return obj
         # fix Blender shorten the name
         # ie. 'torso_lift_armature' -> 'torso_lift_armatu.000'
-        test_prefix = name[:17] + '.'
+        test_prefix = name.rsplit('.')[0]
         # look for candidates
         candidates = [obj for obj in objects \
                       if obj.name.startswith(test_prefix)]
@@ -252,8 +266,12 @@ class AbstractComponent(object):
                 logger.warning(test_prefix + ": more than 1 candidate: " + \
                                str(candidates))
             return candidates[0]
-        else:
-            logger.warning(test_prefix + ": no candidate in " + str(objects))
+        
+        # nothing found yet. Start to search recursively:
+        if recursive:
+            for obj in objects:
+                found = self.get_child(name, obj.children, recursive)
+                if found: return found
 
         return None
 
@@ -261,7 +279,7 @@ class AbstractComponent(object):
         logger.warning("configure_mw is deprecated, use add_stream instead")
         return self.add_stream(datastream, method, path, component)
 
-    def add_stream(self, datastream, method=None, path=None, classpath=None, **kwargs):
+    def add_stream(self, datastream, method=None, path=None, classpath=None, direction = None, **kwargs):
         """ Add a data stream interface to the component
 
         Do the binding between a component and the method to export/import its
@@ -284,80 +302,111 @@ class AbstractComponent(object):
         if not classpath:
             classpath = self.property_value("classpath")
 
+        if not classpath:
+            logger.error("%s: no classpath defined for this "
+                         "component! Check component definition " % self.name)
+            return
+
         level = self.property_value("abstraction_level") or "default"
 
         config = []
         # Configure the datastream for this component
         if not method:
             if not classpath in MORSE_DATASTREAM_DICT:
-                logger.error("%s: no interfaces available for this component!"
-                             "Check builder/data.py." % classpath)
-                return
 
-            interfaces = MORSE_DATASTREAM_DICT[classpath]
-            if not level in interfaces:
+                # Check if we can use default interface...
+                from morse.core.actuator import Actuator
+                from morse.core.sensor import Sensor
+                klass = get_class(classpath)
+                if klass and \
+                   issubclass(klass, Actuator) and \
+                   datastream in INTERFACE_DEFAULT_IN:
 
-                if level == "default": # we need to look for the default level
-                    module_name, class_name = classpath.rsplit('.', 1)
-                    klass = load_module_attribute(module_name, class_name)
+                    logger.warning("%s: no interfaces available for this "
+                                   "component! Trying to use default one "
+                                   "for %s." % (classpath, datastream))
+                    config = [INTERFACE_DEFAULT_IN[datastream]]
 
-                    if not hasattr(klass, "_levels"):
-                        logger.error("Component <%s> does not declare any "
-                                     "default interface. You must call "
-                                     "`add_stream` with an explicit method "
-                                     "and Python module." % str(classpath))
-                        return
+                elif klass and \
+                     issubclass(klass, Sensor) and \
+                     datastream in INTERFACE_DEFAULT_OUT:
 
-                    # iterate over levels to find the one with the default flag
-                    for key, value in klass._levels.items():
-                        if value[2] == True:
-                            level = key
-                            # set the right default level
-                            self.properties(abstraction_level = level)
-                            logger.info("Using default level <%s> for "
-                                        "component <%s>" % (level, classpath))
-                            break
-
-                    if level == "default":
-                        logger.error("Component <%s> does not declare any"
-                                     "default interface, and none of its "
-                                     "abstraction levels is marked as the "
-                                     "default one. You must call `add_stream`"
-                                     " with an explicit method and Python "
-                                     "module." % str(classpath))
-                        return
-
-                    if not level in interfaces:
-                        logger.error("%s: no interfaces defined for this "
-                                     "component for abstraction level <%s>!"
-                                     "Check builder/data.py." %
-                                     (classpath, level))
-                        return
+                    logger.warning("%s: no interfaces available for this "
+                                   "component! Trying to use default one "
+                                   "for %s." % (classpath, datastream))
+                    config = [INTERFACE_DEFAULT_OUT[datastream]]
 
                 else:
-                    logger.error("%s: no interfaces defined for this component"
-                                 "for abstraction level <%s>! Check "
-                                 "builder/data.py." %
-                                 (classpath, level))
+                    logger.error("%s: no interfaces available for this component!"
+                                " Check builder/data.py." % classpath)
+                    return
 
-            interfaces = interfaces[level]
-            if not datastream in interfaces:
-                logger.error("%s: no %s interface defined for this component "
-                             "for abstraction level <%s>! "
-                             "Check builder/data.py." %
-                             (classpath, datastream, level))
-                return
+            else:
+                interfaces = MORSE_DATASTREAM_DICT[classpath]
 
-            config = interfaces[datastream]
-            if isinstance(config, list):
-                config = config[0]
+                if not level in interfaces:
 
-            if config == INTERFACE_DEFAULT_OUT:
-                config = INTERFACE_DEFAULT_OUT[datastream]
-            if config == INTERFACE_DEFAULT_IN:
-                config = INTERFACE_DEFAULT_IN[datastream]
-            if isinstance(config, str):
-                config = [config]
+                    if level == "default": # we need to look for the default level
+                        module_name, class_name = classpath.rsplit('.', 1)
+                        klass = load_module_attribute(module_name, class_name)
+
+                        if not hasattr(klass, "_levels"):
+                            logger.error("Component <%s> does not declare any "
+                                        "default interface. You must call "
+                                        "`add_stream` with an explicit method "
+                                        "and Python module." % str(classpath))
+                            return
+
+                        # iterate over levels to find the one with the default flag
+                        for key, value in klass._levels.items():
+                            if value[2] == True:
+                                level = key
+                                # set the right default level
+                                self.properties(abstraction_level = level)
+                                logger.info("Using default level <%s> for "
+                                            "component <%s>" % (level, classpath))
+                                break
+
+                        if level == "default":
+                            logger.error("Component <%s> does not declare any"
+                                        "default interface, and none of its "
+                                        "abstraction levels is marked as the "
+                                        "default one. You must call `add_stream`"
+                                        " with an explicit method and Python "
+                                        "module." % str(classpath))
+                            return
+
+                        if not level in interfaces:
+                            logger.error("%s: no interfaces defined for this "
+                                        "component for abstraction level <%s>!"
+                                        "Check builder/data.py." %
+                                        (classpath, level))
+                            return
+
+                    else:
+                        logger.error("%s: no interfaces defined for this component"
+                                    "for abstraction level <%s>! Check "
+                                    "builder/data.py." %
+                                    (classpath, level))
+
+                interfaces = interfaces[level]
+                if not datastream in interfaces:
+                    logger.error("%s: no %s interface defined for this component "
+                                "for abstraction level <%s>! "
+                                "Check builder/data.py." %
+                                (classpath, datastream, level))
+                    return
+
+                config = interfaces[datastream]
+                if isinstance(config, list):
+                    config = config[0]
+
+                if config == INTERFACE_DEFAULT_OUT:
+                    config = INTERFACE_DEFAULT_OUT[datastream]
+                if config == INTERFACE_DEFAULT_IN:
+                    config = INTERFACE_DEFAULT_IN[datastream]
+                if isinstance(config, str):
+                    config = [config]
 
         elif not path:
             config = [method]
@@ -397,11 +446,13 @@ class AbstractComponent(object):
         self.add_service(interface)
         self.add_stream(interface, **kwargs)
 
-    def alter(self, modifier_name, config=None, **kwargs):
+    def alter(self, modifier_name, classpath=None, **kwargs):
         """ Add a modifier specified by its first argument to the component """
         # Configure the modifier for this component
-        if not config:
-            config = MORSE_MODIFIER_DICT[modifier_name][self._blender_filename]
+        config = []
+        if not classpath:
+            classpath = MORSE_MODIFIER_DICT[modifier_name][self._blender_filename]
+        config.append(classpath)
         config.append(kwargs)
         Configuration.link_modifier(self, config)
 
@@ -526,22 +577,27 @@ class AbstractComponent(object):
             filepath = os.path.join(MORSE_COMPONENTS, self._category, \
                                     component + '.blend')
 
+        looked_dirs = [filepath]
+
         if not os.path.exists(filepath):
             # Search for some blend file in different paths
             filepath = None
-            resource_path = MORSE_RESOURCE_PATH.split(':')
+            resource_path = MORSE_RESOURCE_PATH.split(os.pathsep)
             for path in resource_path:
                 tmp = os.path.join(path, component)
+                looked_dirs.append(tmp)
                 if os.path.exists(tmp):
                     filepath = tmp
                     break
             # Check if we got a match
+
             if not filepath:
-                logger.error("Blender file '%s' for external asset import can" \
-                             "not be found.\nEither provide an absolute path," \
-                             " or a path relative to MORSE assets directory\n" \
-                             "(typically $PREFIX/share/morse/data)"%component)
-                return
+                logger.error("Error while trying to load '%s': model not found.\n"
+                             "I was looking for one of these files: \n%s\n"
+                             "Either provide an absolute path, or a path relative \n"
+                             "to MORSE assets directories ($MORSE_RESOURCE_PATH \n"
+                             "or default path, typically $PREFIX/share/morse/data)."% (component, looked_dirs))
+                raise FileNotFoundError("%s '%s' not found"%(self.__class__.__name__, component))
 
         if not objects: # link_append all objects from blend file
             objects = bpymorse.get_objects_in_blend(filepath)
@@ -591,6 +647,30 @@ class AbstractComponent(object):
 
         return imported_objects
 
+    def _make_transparent(self, obj, alpha):
+        obj.game.physics_type = 'NO_COLLISION'
+        for m in obj.material_slots:
+            m.material.use_transparency = True
+            m.material.alpha = alpha
+            m.material.transparency_method = 'Z_TRANSPARENCY'
+        for c in obj.children:
+            self._make_transparent(c, alpha)
+
+    def _set_color(self, obj, rgb):
+        if len(obj.material_slots) > 0:
+            obj.active_material.diffuse_color = rgb
+        for c in obj.children:
+            self._set_color(c, rgb)
+
+    def set_color(self, r, g, b):
+        """ Set the color of the component.
+        
+        Will try to apply the RGB color to the active material
+        of all the meshes that are children of this component.
+        """
+        if self._bpy_object:
+            self._set_color(self._bpy_object, (r,g,b))
+
     def profile(self):
         """ Watch the average time used during the simulation.
 
@@ -601,7 +681,7 @@ class AbstractComponent(object):
             logger.warning("profile currently supports only sensors (%s)"%self)
         for key in ["profile", "profile_action", "profile_modifiers",
                     "profile_datastreams"]:
-            prop = self._property_new(key, "0")
+            prop = bpymorse._property_new(self._bpy_object, key, "0")
             prop.show_debug = True
         bpymorse.get_context_scene().game_settings.show_debug_properties = True
 
