@@ -13,7 +13,7 @@ class Environment(Component):
     """
     multinode_distribution = dict()
 
-    def __init__(self, filename, fastmode = False):
+    def __init__(self, filename, fastmode = False, component_renaming = True):
         """
         :param fastmode: (default: False) if True, disable most visual
                          effects (like lights...) to get the fastest
@@ -26,8 +26,11 @@ class Environment(Component):
         Component.__init__(self, 'environments', filename)
         AbstractComponent.components.remove(self) # remove myself from the list of components to ensure my destructor is called
 
+        self._handle_default_interface()
+
         # Rename the components according to their variable names
-        self._rename_components()
+        if component_renaming:
+            self._rename_components()
 
         self.fastmode = fastmode
 
@@ -38,13 +41,20 @@ class Environment(Component):
         self._multinode_configured = False
         self._display_camera = None
         self.is_material_mode_custom = False
+        # Add empty object holdings MORSE Environment's properties
+        # for UTM modifier configutation ( uses env.properties(...) )
+        bpymorse.deselect_all()
+        bpymorse.add_morse_empty()
+        obj = bpymorse.get_context_object()
+        obj.name = 'MORSE.Properties'
+        self.set_blender_object(obj)
+        # Init. camera's properties
+        self.set_camera_speed()
+        self.set_camera_clip()
 
-        # define 'Scene_Script_Holder' as the blender object of Enrivonment
-        if not 'Scene_Script_Holder' in bpymorse.get_objects():
-            # Add the necessary objects
-            base = Component('props', 'basics')
-        self.set_blender_object(bpymorse.get_object('Scene_Script_Holder'))
-        # Write the name of the 'environment file'
+    def is_internal_camera(self, camera):
+        return not self._multinode_configured or \
+            camera._bpy_object.parent.name in self.multinode_distribution[node_name]
 
     def _write_multinode(self, node_name):
         """ Configure this node according to its name
@@ -82,7 +92,7 @@ class Environment(Component):
                         'server_address': self._server_address,
                         'server_port': self._server_port,}
         # Create the config file if it does not exist
-        if not 'multinode_config.py' in bpymorse.get_texts.keys():
+        if not 'multinode_config.py' in bpymorse.get_texts().keys():
             bpymorse.new_text()
             bpymorse.get_last_text().name = 'multinode_config.py'
         cfg = bpymorse.get_text('multinode_config.py')
@@ -98,35 +108,54 @@ class Environment(Component):
         and only the hierarchy is added to the name.
         """
 
-        import inspect
-        try:
-            frame = inspect.currentframe()
-            builderscript_frame = inspect.getouterframes(frame)[2][0] # parent frame
+        AbstractComponent.close_context(3)
 
-            for name, component in builderscript_frame.f_locals.items():
-                if isinstance(component, AbstractComponent):
+        for component in AbstractComponent.components:
+            if isinstance(component, Robot):
 
-                    if hasattr(component, "parent"):
-                        continue
+                def renametree(cmpt, fqn):
+                    if not cmpt.basename:
+                        return
+#                        raise SyntaxError("You need to assign the component of type %s to a variable" %
+#                                        cmpt)
+                    list_name = [cmpt._bpy_object.name for cmpt in AbstractComponent.components if cmpt != component ]
+                    fqn.append(cmpt.basename)
+                    new_name = '.'.join(fqn)
 
-                    if not component.basename: # do automatic renaming only if a name is not already manually set
-                        component.basename = name
-
-                    def renametree(cmpt, fqn):
-                        if not cmpt.basename:
-                            raise SyntaxError("You need to assign the component of type %s to a variable" %
-                                             cmpt)
-                        fqn.append(cmpt.basename)
+                    i = 1
+                    while new_name in list_name:
+                        fqn.pop()
+                        fqn.append("%s_%03d" % (cmpt.basename, i))
                         new_name = '.'.join(fqn)
-                        Configuration.update_name(cmpt.name, new_name)
-                        cmpt._bpy_object.name = '.'.join(fqn)
-                        for child in cmpt.children:
-                            renametree(child, fqn[:])
+                        i += 1
+
+                    Configuration.update_name(cmpt.name, new_name)
+                    cmpt._bpy_object.name = new_name
+                    for child in cmpt.children:
+                        renametree(child, fqn[:])
+
+                renametree(component, [])
+
+    def _handle_default_interface(self):
+        """
+        Handle the semantic of default interface.
+
+        For each robot, for each of this children, if the child has no
+        specific configuration for its 'default_interface', the function
+        adds it automatically.
+        """
+        for component in AbstractComponent.components:
+            if isinstance(component, Robot) and component.default_interface:
+                for child in component.children:
+                    if child.is_morseable():
+                        if not Configuration.has_datastream_configuration(
+                                child, component.default_interface):
+                            child.add_stream(component.default_interface)
+                        if not Configuration.has_service_configuration(
+                                child, component.default_interface):
+                            child.add_service(component.default_interface)
+
 
-                    renametree(component, [])
-        finally:
-            del builderscript_frame
-            del frame
 
     def place_camera(self, location):
         """ Set the location of the default camera.
@@ -158,10 +187,41 @@ class Environment(Component):
         :param clip_start: Camera near clipping distance, float in meters (default 0.1)
         :param clip_end: Camera far clipping distance, float in meters (default 100)
         """
-        camera_fp = bpymorse.get_object('CameraFP')
-        # camera_fp.data holds the bpy.types.Camera instance
-        camera_fp.data.clip_start = clip_start
-        camera_fp.data.clip_end = clip_end
+        self._camera_clip_start = clip_start
+        self._camera_clip_end   = clip_end
+
+    def set_camera_speed(self, speed=2.0):
+        """ Set the simulator's Camera speed
+
+        :param speed: desired speed of the camera, in meter by second.
+        """
+        self._camera_speed = speed
+
+    def _cfg_camera_scene(self):
+        scene = bpymorse.get_context_scene()
+        scene.name = 'S.MORSE_ENV'
+        from morse.builder.sensors import VideoCamera
+        cfg_camera_scene = {}
+        for component in AbstractComponent.components:
+            # do not create scene for external camera
+            if isinstance(component, VideoCamera) and \
+                    self.is_internal_camera(component):
+                # Create a new scene for the Camera
+                bpymorse.new_scene(type='LINK_OBJECTS')
+                scene = bpymorse.get_context_scene()
+                scene.name = 'S.%s'%component.name
+                scene.render.resolution_x = component.property_value('cam_width')
+                scene.render.resolution_y = component.property_value('cam_height')
+                # TODO disable logic and physic in this created scene
+                cfg_camera_scene[component.name] = scene.name
+
+        # Create 'camera_scene.py' configuration file (like 'component_config.py')
+        # mapping Camera -> Scene for the bge.texture.ImageRender(scene, camera)
+        bpymorse.new_text()
+        bpymorse.get_last_text().name = 'camera_scene.py'
+        cfg = bpymorse.get_text('camera_scene.py')
+        cfg.write('camera_scene = ' + json.dumps(cfg_camera_scene, indent=1) )
+        cfg.write('\n')
 
     def create(self, name=None):
         """ Generate the scene configuration and insert necessary objects
@@ -169,64 +229,83 @@ class Environment(Component):
         Should always be called at the very end of the Builder script. It will
         finalise the building process and write the configuration files.
         """
-        try:
-            # Invoke special methods of component that must take place *after* renaming
-            for component in AbstractComponent.components:
-                if hasattr(component, "after_renaming"):
-                    component.after_renaming()
+        # Invoke special methods of component that must take place *after* renaming
+        for component in AbstractComponent.components:
+            if hasattr(component, "after_renaming"):
+                component.after_renaming()
 
+        # Create a new scene for each camera, with specific render resolution
+        # Must be done at the end of the builder script, after renaming
+        # and before adding 'Scene_Script_Holder'
+        self._cfg_camera_scene()
 
+        # Create a new scene for the MORSE_LOGIC (Scene_Script_Holder, CameraFP)
+        bpymorse.new_scene(type='LINK_OBJECTS')
+        scene = bpymorse.get_context_scene()
+        scene.name = 'S.MORSE_LOGIC'
 
-            # Default node name
-            if name == None:
-                try:
-                    name = os.environ["MORSE_NODE"]
-                except KeyError:
-                    name = os.uname()[1]
-            # Write the configuration of the datastreams, and node configuration
-            Configuration.write_config()
-            self._write_multinode(name)
+        # define 'Scene_Script_Holder' as the blender object of Enrivonment
+        if not 'Scene_Script_Holder' in bpymorse.get_objects():
+            # Add the necessary objects
+            base = Component('props', 'basics')
 
-            # Change the Screen material
-            if self._display_camera:
-                self._set_scren_mat()
+        # Set Scene_Script_Holder as core Environment object
+        self.set_blender_object(bpymorse.get_object('Scene_Script_Holder'))
+        # Copy properties (for UTM modifier configuration)
+        _properties = bpymorse.get_properties(bpymorse.get_object('MORSE.Properties'))
+        self.properties(**_properties)
 
-            self.properties(environment_file = str(self._environment_file))
-            # Set the position of the camera
-            camera_fp = bpymorse.get_object('CameraFP')
-            camera_fp.location = self._camera_location
-            camera_fp.rotation_euler = self._camera_rotation
+        # Default node name
+        if name == None:
+            try:
+                name = os.environ["MORSE_NODE"]
+            except KeyError:
+                name = os.uname()[1]
+        # Write the configuration of the datastreams, and node configuration
+        Configuration.write_config()
+        self._write_multinode(name)
 
-            if self.fastmode:
-                self.set_material_mode('SINGLETEXTURE')
-                self.set_viewport("WIREFRAME")
-            elif not self.is_material_mode_custom:
-                # make sure OpenGL shading language shaders (GLSL) is the
-                # material mode to use for rendering
-                self.set_material_mode('GLSL')
+        # Change the Screen material
+        if self._display_camera:
+            self._set_scren_mat()
 
-            # Set the unit system to use for button display (in edit mode) to metric
-            bpymorse.get_context_scene().unit_settings.system = 'METRIC'
-            # Select the type of Framing to Extend,
-            # Show the entire viewport in the display window,
-            # viewing more horizontally or vertically.
-            bpymorse.get_context_scene().game_settings.frame_type = 'EXTEND'
-            # Start player with a visible mouse cursor
-            bpymorse.get_context_scene().game_settings.show_mouse = True
+        # Write the name of the 'environment file'
+        self.properties(environment_file = str(self._environment_file))
 
-            # Make CameraFP the active camera
-            bpymorse.deselect_all()
-            camera_fp.select = True
-            bpymorse.get_context_scene().objects.active = camera_fp
-            # Set default camera
-            bpymorse.get_context_scene().camera = camera_fp
+        if self.fastmode:
+            self.set_material_mode('SINGLETEXTURE')
+            self.set_viewport("WIREFRAME")
+        elif not self.is_material_mode_custom:
+            # make sure OpenGL shading language shaders (GLSL) is the
+            # material mode to use for rendering
+            self.set_material_mode('GLSL')
 
-            self._created = True
-        except BaseException:
-            logger.error("Your MORSE Builder script is invalid!")
-            import traceback
-            traceback.print_exc()
-            os._exit(-1)
+        # Set the unit system to use for button display (in edit mode) to metric
+        bpymorse.get_context_scene().unit_settings.system = 'METRIC'
+        # Select the type of Framing to Extend,
+        # Show the entire viewport in the display window,
+        # viewing more horizontally or vertically.
+        bpymorse.get_context_scene().game_settings.frame_type = 'EXTEND'
+        # Start player with a visible mouse cursor
+        bpymorse.get_context_scene().game_settings.show_mouse = True
+
+        # Set the position of the camera
+        camera_fp = bpymorse.get_object('CameraFP')
+        camera_fp.location = self._camera_location
+        camera_fp.rotation_euler = self._camera_rotation
+        camera_fp.game.properties['Speed'].value = self._camera_speed
+        camera_fp.data.clip_start = self._camera_clip_start
+        camera_fp.data.clip_end   = self._camera_clip_end
+        # Make CameraFP the active camera
+        bpymorse.deselect_all()
+        camera_fp.select = True
+        bpymorse.get_context_scene().objects.active = camera_fp
+        # Set default camera
+        bpymorse.get_context_scene().camera = camera_fp
+        # Set viewport to Camera
+        bpymorse.set_viewport_perspective()
+
+        self._created = True
 
     def set_horizon_color(self, color=(0.05, 0.22, 0.4)):
         """ Set the horizon color
@@ -285,16 +364,12 @@ class Environment(Component):
         bpymorse.get_context_scene().game_settings.material_mode = material_mode
         self.is_material_mode_custom = True
 
-    def set_viewport(self, viewport_shade='WIREFRAME'):
+    def set_viewport(self, viewport_shade='WIREFRAME', clip_end=1000):
         """ Set the default view mode
 
         :param viewport_shade: enum in ['BOUNDBOX', 'WIREFRAME', 'SOLID', 'TEXTURED'], default 'WIREFRAME'
         """
-        for area in bpymorse.get_context_window().screen.areas:
-            if area.type == 'VIEW_3D':
-                for space in area.spaces:
-                    if space.type == 'VIEW_3D':
-                        space.viewport_shade = viewport_shade
+        bpymorse.set_viewport(viewport_shade, clip_end)
 
     def set_auto_start(self, auto_start=True):
         bpymorse.get_context_scene().render.engine = 'BLENDER_GAME'
@@ -420,14 +495,32 @@ class Environment(Component):
         # Change the text with the name of the camera being displayed
         caption.game.properties['Text'].value = self._display_camera.name
 
-    def save(self, filepath=None, check_existing=False):
+    def save(self, filepath=None, check_existing=False, compress=True):
         """ Save .blend file
 
-        :param filepath: (string, (optional, default: current file)) File Path
-        :param check_existing: (boolean, (optional, default: False))
-                               Check and warn on overwriting existing files
+        :param filepath: File Path
+        :type  filepath: string, (optional, default: current file)
+        :param check_existing: Check and warn on overwriting existing files
+        :type  check_existing: boolean, (optional, default: False)
+        :param compress: Compress, Write compressed .blend file
+        :type  compress: boolean, (optional, default: True)
         """
-        bpymorse.save(filepath=filepath, check_existing=check_existing)
+        bpymorse.save(filepath=filepath, check_existing=check_existing,
+                compress=compress)
+
+    def set_log_level(self, component, level):
+        """ 
+        Set the debug level of the component to the level level.
+
+        :param component: the class name of the component
+        :param level: a string representing the level of debug to set
+
+        XXX not persistent (if you save the scene and reload it directly
+        from blender, the logger is not set at the right level.
+        """
+
+        my_logger = logging.getLogger('morse.' + component)
+        my_logger.setLevel(level.upper())
 
     def __del__(self):
         """ Call the create method if the user has not explicitly called it """
