@@ -320,6 +320,14 @@ class Infrared(LaserSensorWithArc):
         # create default Arc_
         self.create_laser_arc()
 
+class Velocity(SensorCreator):
+    def __init__(self, name=None):
+        SensorCreator.__init__(self, name, "morse.sensors.velocity.Velocity", "velocity")
+        mesh = Sphere("VelocitySphere")
+        mesh.scale = (.04, .04, .01)
+        mesh.color(.5, .5, .5)
+        self.append(mesh)
+
 class VideoCamera(SensorCreator):
     def __init__(self, name=None, \
                  classpath="morse.sensors.video_camera.VideoCamera", \
@@ -348,6 +356,7 @@ class VideoCamera(SensorCreator):
         SensorCreator.rotate(self, x=math.pi/2, z=math.pi/2)
         # append CameraMesh with its textures
         self.append_meshes(['CameraMesh'], "camera")
+        self.rotate(z=math.pi)
     def rotate(self, x=0, y=0, z=0):
         SensorCreator.rotate(self, x=y, y=z, z=x)
 
@@ -359,7 +368,25 @@ class DepthCamera(VideoCamera):
         self.properties(cam_width = 128, cam_height = 128, \
                         cam_near=1.0, cam_far=20.0, retrieve_depth=True,
                         Vertical_Flip=False)
-        self.camera.rotate(z=math.pi)
+
+class VelodyneZB(DepthCamera):
+    def __init__(self, name=None):
+        DepthCamera.__init__(self, name)
+        # Always (MORSE_LOGIC) - And - Motion (rotation z: 0.017453)
+        morselogic = [sensor for sensor in self._bpy_object.game.sensors \
+                      if sensor.name.startswith('MORSE_LOGIC')]
+        sensor = morselogic[0]
+        bpymorse.add_controller(type='LOGIC_AND')
+        controller = self._bpy_object.game.controllers[-1]
+        # Motion (rotation z: 0.017453)
+        bpymorse.add_actuator(type='MOTION', name='MORSE_ROTATE')
+        actuator = self._bpy_object.game.actuators[-1]
+        actuator.offset_rotation.y = self.camera._bpy_object.data.angle # FOV
+        controller.link(sensor = sensor, actuator = actuator)
+    def set_rotation(self, rotation):
+        """set rotation per tic (in radian, relative to component frequency)"""
+        actuator = self._bpy_object.game.actuators['MORSE_ROTATE']
+        actuator.offset_rotation.y = rotation
 
 class SemanticCamera(VideoCamera):
     def __init__(self, name=None):
@@ -413,3 +440,36 @@ class Kinect(SensorCreator):
         # Override AbstractComponent method
         self.video_camera.profile()
         self.depth_camera.profile()
+
+class Collision(SensorCreator):
+    def __init__(self, name=None):
+        """ Sensor to detect objects colliding with the current object,
+        with more settings than the Touch sensor
+        """
+        SensorCreator.__init__(self, name)
+        self.properties(classpath = "morse.sensors.collision.Collision")
+        obj = bpymorse.get_context_object()
+        # Sensor, Collision Sensor, detects static and dynamic objects but
+        # not the other collision sensor objects.
+        obj.game.physics_type = 'SENSOR'
+        # Specify a collision bounds type other than the default
+        obj.game.use_collision_bounds = True
+        # replace Always sensor by Collision sensor
+        sensor = obj.game.sensors[-1]
+        sensor.type = 'COLLISION'
+        # need to get the new Collision Sensor object
+        sensor = obj.game.sensors[-1]
+        sensor.use_pulse_true_level = True # FIXME doesnt seems to have any effect
+        # Component mesh (eye sugar)
+        mesh = Cube("CollisionMesh")
+        mesh.scale = (.02, .02, .02)
+        mesh.color(.8, .2, .1)
+        self.append(mesh)
+    def properties(self, **kwargs):
+        SensorCreator.properties(self, **kwargs)
+        if 'collision_property' in kwargs:
+            try:
+                sensor = self._bpy_object.game.sensors[-1]
+                sensor.property = kwargs['collision_property']
+            except KeyError:
+                pass
